package com.crudoperation.Service;

import com.crudoperation.Model.Course;
import com.crudoperation.Model.Student;
import com.crudoperation.Repository.CourseRepository;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.junit.jupiter.MockitoExtension;

import java.util.stream.Collectors;
import java.util.stream.Stream;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

@ExtendWith(MockitoExtension.class)
class CourseServiceImplTest {

    @InjectMocks private CourseServiceImpl   courseServiceImpl;

    @Mock private CourseRepository courseRepository;


    @Test
    void getAllCourses() {

        when(courseRepository.findAll()).thenReturn(Stream.of(new Course(1, "Java"), new Course(2, "Python")).collect(Collectors.toList()));
        assertEquals(2, courseServiceImpl.getAllCourses().size());
    }


    @Test
    void getCourseByID() {
        Course course = new Course(3, "JavaScript");
        when(courseRepository.findById(3)).thenReturn(course);
        Course courseResult = courseServiceImpl.getCourseByID(3);
        assertEquals(course, courseResult);
    }

    @Test
    void addCourse() {
        Course courseadd = new Course(4, "PHP");
        when(courseRepository.save(courseadd)).thenReturn(courseadd);
        Course courseadded = courseServiceImpl.addCourse(courseadd);
        assertEquals(courseadd, courseadd);
    }
//
    @Test
    void deleteCourseById() {

        Course coursedelete = new Course(5, "JQuery");
        courseServiceImpl.deleteCourseById(coursedelete.getCid());
        verify(courseRepository, times(1)).deleteById(5);
    }

    @Test
    void updateCourse() {

        Course course = new Course();
        course.setCid(6);
        course.setCname("Automation");
        Mockito.when(courseRepository.save(course)).thenReturn(null);
        course.setCname("C++");
        Mockito.when(courseRepository.save(course)).thenReturn(course);
        assertEquals(course, courseServiceImpl.updateCourse(course, course.getCid()));

    }
}